<?php

namespace Drupal\beehotel_pricealterator\Controller;

use Drupal\beehotel_pricealterator\PriceAlteratorPluginManager;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\Markup;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A controller for a price Alterator list.
 */
class Alterators extends ControllerBase {

  /**
   * Drupal configuration service container.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * The route provider.
   *
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected $routeProvider;

  /**
   * The plugin manager.
   *
   * @var \Drupal\beehotel_pricealterator\PriceAlteratorPluginManager
   */
  protected $priceAlteratorPluginManager;

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The config factory.
   * @param \Drupal\Core\Routing\RouteProviderInterface $provider
   *   The route provider.
   * @param \Drupal\beehotel_pricealterator\PriceAlteratorPluginManager $price_alterator_plugin_manager
   *   The tempstore factory.
   */
  public function __construct(ConfigFactory $config_factory, RouteProviderInterface $provider, PriceAlteratorPluginManager $price_alterator_plugin_manager) {
    $this->configFactory = $config_factory;
    $this->routeProvider = $provider;
    $this->priceAlteratorPluginManager = $price_alterator_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('router.route_provider'),
      $container->get('plugin.manager.beehotel.pricealterator'),
    );
  }

  /**
   * Returns a list of price alterators.
   *
   * @return array
   *   A renderable array.
   */
  public function alteratorsList() {

    $alterators = $this->priceAlteratorPluginManager->sortedAlterators();

    $rows = [];

    foreach ($alterators as $k => $a) {

      $settings = "";

      // @todo replace this with https://www.drupal.org/project/drupal/issues/3029545
      $route = 'beehotel_pricealterators.' . strtolower($k) . '_settings';
      $exists = count($this->routeProvider->getRoutesByNames([$route])) === 1;

      if ($exists == 1) {
        $url = Url::fromRoute('beehotel_pricealterators.' . strtolower($k) . '_settings');
        $settings = Link::fromTextAndUrl($this->t("Settings"), $url)->toString();
      }

      $config = $this->configFactory
        ->getEditable('beehotel_pricealterators.' . $a['id'] . '.settings');

      if ($a['status'] == 0) {
        $a['status'] = Markup::create("<span class='disabled'>NA</span>");
      }
      elseif ($a['status'] == 1 && $config->get('status') == 1) {
        $a['status'] = Markup::create("<span class='enabled'>ON</span>");
      }
      elseif ($a['type'] == "optional" && $a['status'] == 1 && $config->get('status') == 0) {
        $a['status'] = Markup::create("<span class='disabled'>OFF</span>");
      }
      elseif ($a['status'] == 1) {
        $a['status'] = Markup::create("<span class='enabled'>AV</span>");
      }

      $mandatory = ($a['type'] == 'mandatory') ? '<span class="mandatory">*</span>' : '';

      $rows[] = [
        Markup::create('<strong>' . $a['id'] . '</strong>' . $mandatory),
        $a['description'],
        $a['weight'],
        $a['status'],
        $settings,
      ];

    }

    $header = [
      'title' => $this->t('Id'),
      'description' => $this->t('Description'),
      'weight' => $this->t('Weight'),
      'status' => $this->t('Status'),
      'settings' => $this->t('Settings'),
    ];

    $url = Url::fromUri('https://www.drupal.org/node/3333744');

    $build['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No content has been found.'),
      '#caption' => $this->t('Price Alterators on this system'),
    ];

    $tmp = $this->t("Your own Algorithm to produce smart prices from rules you define");
    $title = "";
    $description = $this->t("<h5>The BEE Hotel Algorithm will produce smart prices for your Hotel from rules you define with Alterators.</h5>") . "<br>";
    $description .= $this->t("The price Algorithm is a chain of Alterators that dynamically produces prices for your hospitality business according to a logic you define.") . "<br>";
    $description .= $this->t("In this page you can enable and set up existing alterators provided by the <a target='_blank' href='https://www.drupal.org/project/bee_hotel'>BEE Hotel module</a>. If your business needs more specific Alterators, you can add your own.") . "<br>";
    $description .= $this->t("Read the price Algorithm Doc page at <a target='_blank' href='https://www.drupal.org/node/3354393'>https://www.drupal.org/node/3354393</a>") . "<br>";
    $description .= '<br/>For support, ask the <a href="https://www.drupal.org/community">developer community</a> or the <a href="https://www.drupal.org/u/afagioli">module author</a><br>';
    $description .= '<h4>Table legenda</h4>';
    $description .= '<ul>';
    $description .= '<li>Id: Alterator Id';
    $description .= '<li><span class="mandatory">*</span>: mandatory';
    $description .= '<li>Description a string to define what this Alterators does';
    $description .= '<li>Weight: used by Drupal to sort Alterators';
    $description .= '<li>Status:';
    $description .= '<ul>';
    $description .= '<li>NA: Not Available, by code';
    $description .= '<li>AV: Available, by code';
    $description .= '<li>OFF: switched off, click settings to enable';
    $description .= '<li>ON: switched on, click settings to disable';
    $description .= '</ul>';
    $description .= '<li>Settings: a link to the Alterator setup form';
    $description .= '</ul>';

    $details = [
      '#type' => 'details',
      '#title' => $tmp,
      '#description' => $description,
      '#open' => FALSE,
    ];

    return [
      '#type' => '#markup',
      '#markup' => $title . " " . render($details) . render($build),
    ];
  }

}
