<?php

namespace Drupal\bee_hotel\Controller;

use Drupal\bee_hotel\Util\BeeHotelUnit;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Drupal\commerce_product\Entity\Product;

/**
 * Provides route responses for BeeHotel module.
 */
class RelatedProduct extends ControllerBase {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;


  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilder
   */
  protected $formBuilder;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new Vertical object.
   *
   * @param \Drupal\bee_hotel\Util\BeeHotelUnit $bee_hotel_unit
   *   The BeeHotel Unit Utility.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(BeeHotelUnit $bee_hotel_unit, EntityTypeManagerInterface $entity_type_manager) {
    $this->beehotelunit = $bee_hotel_unit;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Produces a related product page fora given $node.
   */
  public function product(Node $node) {
    $data = [];

    // Check data.
    $this->config->set('unit_reservation_form_disabled', FALSE)->save();

    if ($this->config->get('beehotel.setup_mode') == 1) {
      $this->beehotelunit->checkBeeHotelSetup($node);
    }

    $data['tmp'] = $node->get("field_product")->target_id;
    $data['product'] = Product::load((int) $data['tmp']);

    /*Load Product Variations*/
    foreach ($data['product']->getVariationIds() as $variation) {
      $data['tmp'] = $this->entityTypeManager->getStorage('commerce_product_variation')->load((int) $variation);
      $data['variations']['items'][] = $data['tmp'];
      $data['variations']['html'][] = $data['tmp']->toLink()->toRenderable();
    }

    $data['tmp'] = Url::fromUri("base://product/" . $data['product']->Id() . "/variations", ['absolute' => FALSE])->toString();
    $data['variations']['link'] = $this->t('See <a href="@uri">variation</a> page.', ['@uri' => $data['tmp']]);

    $data['title'] = $data['product']->toLink()->toRenderable();

    $output = "<h2>" . $this->t("Product and variations related to this Bee Hotel unit") . "</h2>";
    $output .= "<p>" . $this->t("Product:") . " <b>" . render($data['title']) . "</b></p>";
    $output .= "<p>" . $this->t("Variations:") . "</p>";

    if (isset($data['variations']['html'])) {
      $output .= "<ol>";
      foreach ($data['variations']['html'] as $html) {
        $output .= "<li>" . render($html);
      }
      $output .= "</ol>";
    }

    $output .= $data['variations']['link'];
    $build = ['#markup' => $output];

    return $build;
  }

}
