<?php

namespace Drupal\bee_hotel\Util;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Drupal\commerce_product\Entity\ProductAttribute;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\commerce_product\ProductAttributeFieldManagerInterface;

/**
 * Utilities for BeeHotel.
 */
class BeeHotel implements ContainerInjectionInterface {

  /**
   * The attribute field manager.
   *
   * @var \Drupal\commerce_product\ProductAttributeFieldManagerInterface
   */
  protected $attributeFieldManager;

  /**
   * The bee hotel unit.
   *
   * @var \Drupal\bee_hotel\Util\BeeHotelUnit
   */
  private $beehotelunit;

  /**
   * Constructs a new ProductAttributeForm object.
   *
   * @param \Drupal\commerce_product\ProductAttributeFieldManagerInterface $attribute_field_manager
   *   The attribute field manager.
   * @param \Drupal\bee_hotel\Util\BeeHotelUnit $bee_hotel_unit
   *   The BeeHotel Unit Utility.
   */
  public function __construct(ProductAttributeFieldManagerInterface $attribute_field_manager, BeeHotelUnit $bee_hotel_unit) {
    $this->attributeFieldManager = $attribute_field_manager;
    $this->beehotelunit = $bee_hotel_unit;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('commerce_product.attribute_field_manager'),
      $container->get('bee_hotel.util.beehotelunit'),
    );
  }

  /**
   * Guest SETUP mode.
   */
  public function checkBeeHotelSetup(EntityInterface $node) {

    $tmp = "<p>" . $this->t("Bee Hotel SETUP mode ON. Once set up is done,
      <a href='@path'>turn it OFF</a>", ['@path' => "/admin/beehotel/settings"]);
    $this->messenger()->addWarning(Markup::create($tmp));

    $this->messenger()->addWarning($tmp);

    $this->checkGuestAttribute();

    // Accept reservation.
    $this->beehotelunit->checkUnitAcceptsReservations($node);

    // Correct variations.
    $this->beehotelunit->checkProductVariations($node);

    // Weekly price table.
    $this->beehotelunit->checkBasePriceTable($node);
  }

  /**
   * Guest attribute check.
   */
  public static function checkGuestAttribute() {
    // A. Guests attributes required.
    if (!ProductAttribute::load('guests')) {
      $tmp = t("BEE Hotel requires a commerce  attribute named 'guests', not found in current configuration.") . " ";
      $tmp .= t("<a href='@path'>Fix this</a> now", ['@path' => '/admin/commerce/product-attributes']);
      \Drupal::messenger()->addError(Markup::create($tmp));
    }

    // B. BEE flag as variation type.
    $attribute_field_manager = \Drupal::service('commerce_product.attribute_field_manager');
    $map = $attribute_field_manager->getFieldMap();
    if (!isset($map['bee'])) {
      $link = Link::fromTextAndUrl('Set BEE as "Product variation types"', Url::fromUserInput('/admin/commerce/product-attributes/manage/guests'));
      \Drupal::messenger()->addWarning($link);
    }
  }

}
